# -*- encoding: utf-8 -*-

'''
Author     :Phailin
Datetime   :2023-08-01 09:00:00
E-Mail     :phailin791@hotmail.com
'''

from contextlib import contextmanager, AbstractContextManager
from typing import Callable
from loguru import logger

from sqlalchemy import create_engine, orm
from sqlalchemy.orm import declarative_base, Session, Mapped, mapped_column

Base = declarative_base()

class Database:

    def __init__(self, db_url: str, echo: bool = False) -> None:
        self._engine = create_engine(db_url, echo=echo, pool_pre_ping=True, pool_size=100, max_overflow=0)
        self._session_factory = orm.scoped_session(
            orm.sessionmaker(
                autocommit=False,
                autoflush=False,
                bind=self._engine,
            ),
        )

    @contextmanager
    def session(self) -> Callable[..., AbstractContextManager[Session]]:
        session: Session = self._session_factory()
        try:
            yield session
        except Exception:
            logger.error("Session rollback because of exception")
            session.rollback()
            raise
        finally:
            session.close()


# -*- encoding: utf-8 -*-

'''
Author     :Phailin
Datetime   :2023-08-01 09:00:00
E-Mail     :phailin791@hotmail.com
'''

from jose import jwt
from passlib.context import CryptContext
from datetime import datetime,timedelta
from pydantic import ValidationError

pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")

ALGORITHM = "HS256"

def generate_token(data: dict, expires_delta: timedelta | None = None, secret_key: str | None = None) -> str:
    to_encode = data.copy()
    # valid time
    if not expires_delta:
        expires_delta = 15
    expire = datetime.now() + timedelta(minutes=expires_delta)
    # encode and update
    to_encode.update({"exp": expire})
    # jwt
    encoded_jwt = jwt.encode(to_encode, secret_key, algorithm=ALGORITHM)
    # return
    return encoded_jwt

def parse_token(token: str, secret_key: str | None = None):
    try:
        return jwt.decode(token, secret_key, algorithms=[ALGORITHM])
    except (jwt.JWTError, ValidationError):
        # raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="Could not validate credentials")
        return {"sub": 0}

def verify_password(plain_password: str, hashed_password: str) -> bool:
    return pwd_context.verify(plain_password, hashed_password)

def get_password_hash(password: str) -> str:
    return pwd_context.hash(password)


# -*- encoding: utf-8 -*-

'''
Author     :Phailin
Datetime   :2023-08-01 09:00:00
E-Mail     :phailin791@hotmail.com
'''

from typing import Optional
from datetime import datetime
from sqlalchemy import String
from sqlalchemy.orm import (declared_attr, declarative_mixin, Mapped, mapped_column)

from suzaku_framework.database import Base # noqa

@declarative_mixin
class ModelMixin:

    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()
    
    __table_args__ = {'mysql_engine': 'InnoDB'}
    __mapper_args__= {'always_refresh': True}

    id: Mapped[int] = mapped_column(primary_key=True, sort_order=-1, comment="id")
    created_by: Mapped[Optional[int]] = mapped_column(comment="创建人")
    created_at: Mapped[Optional[datetime]] = mapped_column(comment="创建时间")
    updated_by: Mapped[Optional[int]] = mapped_column(comment="更新人")
    updated_at: Mapped[Optional[datetime]] = mapped_column(onupdate=datetime.now, comment="更新时间")
    comment: Mapped[Optional[str]] = mapped_column(String(64), comment="备注")
    deleted: Mapped[Optional[bool]] = mapped_column(default=False, comment="逻辑删除:False=>未删除,True=>已删除", doc="逻辑删除")


# -*- encoding: utf-8 -*-

'''
Author     :Phailin
Datetime   :2023-08-01 09:00:00
E-Mail     :phailin791@hotmail.com
'''

import sys, os, logging
from loguru import logger
from datetime import datetime
from pprint import pformat
from loguru._defaults import LOGURU_FORMAT

def parse_log_level_value(v):
    if not v:
        v = "DEBUG"
    v = logging.getLevelName(v)
    return v

class InterceptHandler(logging.Handler):
    def emit(self, record):
        # Get corresponding Loguru level if it exists
        try:
            level = logger.level(record.levelname).name
        except ValueError:
            level = record.levelno

        # Find caller from where originated the logged message
        frame, depth = logging.currentframe(), 2
        while frame.f_code.co_filename == logging.__file__:
            frame = frame.f_back
            depth += 1
        
        logger.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())

def format_record(record: dict) -> str:
    """
    Custom format for loguru loggers.
    Uses pformat for log any data like request/response body during debug.
    Works with logging if loguru handler it.
    Example:
    >>> payload = [{"users":[{"name": "Nick", "age": 87, "is_active": True}, {"name": "Alex", "age": 27, "is_active": True}], "count": 2}]
    >>> logger.bind(payload=).debug("users payload")
    >>> [   {   'count': 2,
    >>>         'users': [   {'age': 87, 'is_active': True, 'name': 'Nick'},
    >>>                      {'age': 27, 'is_active': True, 'name': 'Alex'}]}]
    """

    format_string = LOGURU_FORMAT
    if record["extra"].get("payload") is not None:
        record["extra"]["payload"] = pformat(
            record["extra"]["payload"], indent=4, compact=True, width=88
        )
        format_string += "\n<level>{extra[payload]}</level>"

    format_string += "{exception}\n"
    return format_string


def init_logger(project_name, log_level):
    """
    Replaces logging handlers with a handler for using the custom handler.
        
    WARNING!
    if you call the init_logging in startup event function, 
    then the first logs before the application start will be in the old format
    >>> app.add_event_handler("startup", init_logging)
    stdout:
    INFO:     Uvicorn running on http://127.0.0.1:8000 (Press CTRL+C to quit)
    INFO:     Started reloader process [11528] using statreload
    INFO:     Started server process [6036]
    INFO:     Waiting for application startup.
    2020-07-25 02:19:21.357 | INFO     | uvicorn.lifespan.on:startup:34 - Application startup complete.
    
    """

    # disable handlers for specific uvicorn loggers
    # to redirect their output to the default uvicorn logger
    # works with uvicorn==0.11.6
    loggers = (
        logging.getLogger(name)
        for name in logging.root.manager.loggerDict
        if name.startswith("uvicorn.")
    )
    for uvicorn_logger in loggers:
        uvicorn_logger.handlers = []

    # change handler for default uvicorn logger
    intercept_handler = InterceptHandler()
    logging.getLogger("uvicorn.access").handlers = [intercept_handler]

    # set logs output, level and format
    logger.configure(handlers=[{"sink": sys.stdout, "level": logging.DEBUG, "format": format_record}])

    # logger save dir
    logger_dir = os.path.join(os.getcwd(), "logs")
    if not os.path.exists(logger_dir):
        os.makedirs(logger_dir)

    run_time = datetime.now().strftime("%Y_%m_%d")

    logger_file_name = f"{project_name}_{run_time}.log"

    logger_path = os.path.join(logger_dir, logger_file_name)

    log_level = parse_log_level_value(log_level)

    # configure loguru
    logger.add(logger_path, 
        format="{time:YYYY-MM-DD HH:mm:ss.SSS} - [process: {process} - thread: {thread}][{module} - {file}:{line} - {function}] - {level} - {message}", 
        encoding='utf-8', filter="", level=int(log_level), rotation="100 MB", compression="zip", enqueue=True, colorize=True)


# -*- encoding: utf-8 -*-

'''
Author     :Phailin
Datetime   :2023-08-01 09:00:00
E-Mail     :phailin791@hotmail.com
'''

from typing import Dict
from loguru import logger
from suzaku_framework.pathlib_auto.load_module import auto_load_from_group, get_group_file

# dict to object
class DeSerialize:
    def __init__(self, serialized_dict: Dict):
        self._serialized_dict = serialized_dict

    def __getattr__(self, name):
        if name in self._serialized_dict:
            return self._serialized_dict[name]
        raise AttributeError(f"Serialized dict'{name}' not found.")
    
class RegisterMeta:
    def __init__(self, meta_registry: Dict):
        self.meta_registry = meta_registry

    def __call__(self, cls):
        self.meta_registry[cls.__name__] = cls
        # logger.info(f"Registered: {cls.__name__}")
        return cls
    
def print_registered_services(meta_registry: Dict, meta_module: str = None):
    if meta_module:
        logger.info('Load {meta_module} Factory...')
    for service_name, service_class in meta_registry.items():
        if meta_module:
            message = f"Registered {meta_module} {service_name} from {service_class.__module__}"
        else:
            message = f"Registered {service_name} from {service_class.__module__}"
        logger.info(message)

# register meta and load
def load_registered_services_from_meta(meta_registry: Dict, meta_class: str, meta_module: str = None):
    # auto load services
    service_files = get_group_file(meta_class)
    
    # auto load and rigister services
    auto_load_from_group(service_files)
    
    # print registered services
    print_registered_services(meta_registry, meta_module)
    
    # service factory
    return  DeSerialize(meta_registry)

def load_registered_services(meta_registry: Dict, meta_module: str = None):
    # print registered services
    print_registered_services(meta_registry, meta_module)
    
    # service factory and return
    return  DeSerialize(meta_registry)


from distutils.core import setup
from setuptools import find_packages

with open("README.rst", "r" , encoding="utf-8") as f:
    long_description = f.read()
    setup(
        name='suzaku_framework',
        version='0.0.9',
        description="A package for koala's project",
        long_description=long_description,
        author='phailin',
        author_email='phailin791@hotmail.com',
        url='https://github.com/phailin/suzaku_framework',
        install_requires=[
            "fastapi",
            "mysql-connector-python",
            "SQLAlchemy",
            "python-jose[cryptography]",
            "pydantic",
            "pydantic_core",
            "loguru",
            "bcrypt",
            "passlib",
        ],
        license='MIT',
        packages=find_packages(),
        platforms=["all"],
        classifiers=[
            'Intended Audience :: Developers',
            'Operating System :: OS Independent',
            'Natural Language :: Chinese (Simplified)',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.10',
            'Programming Language :: Python :: 3.11',
            'Topic :: Software Development :: Libraries'
        ],
    )